Package: praatpicture
Title: 'Praat Picture' Style Plots of Acoustic Data
Version: 1.9.0
Authors@R: 
    person("Rasmus", "Puggaard-Rode", 
    email = "rasmus.puggaard-rode@ling-phil.ox.ac.uk", 
    role = c("aut", "cre", "cph"),
           comment = c(ORCID = "0000-0003-4522-9987"))
Description: Quickly and easily generate plots of acoustic data aligned with transcriptions similar to those made in 'Praat' using either derived signals generated directly in R with 'wrassp' or imported derived signals from 'Praat'. Provides easy and fast out-of-the-box solutions but also a high extent of  flexibility. Also provides options for embedding audio in figures and animating figures.
License: Apache License (>= 2)
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: av (>= 0.9.0), bslib (>= 0.6.1), crayon (>= 1.5.2), emuR (>=
        2.4.2), gifski (>= 1.12.0.2), graphics (>= 4.3.2), grDevices
        (>= 4.3.2), gsignal (>= 0.3.5), ipa (>= 0.1.0), multitaper (>=
        1.0-17), phonTools (>= 0.2.2.2), rPraat (>= 1.3.2.1),
        rstudioapi (>= 0.15.0), shiny (>= 1.8.1.1), shinyjs (>= 2.1.0),
        soundgen (>= 2.6.2), tuneR (>= 1.4.6), utils (>= 4.3.2), wrassp
        (>= 1.0.4), zoo (>= 1.8-12)
URL: https://github.com/rpuggaardrode/praatpicture
BugReports: https://github.com/rpuggaardrode/praatpicture/issues
NeedsCompilation: no
Packaged: 2026-07-20 15:04:41 UTC; cpgl0183
Author: Rasmus Puggaard-Rode [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0003-4522-9987>)
Maintainer: Rasmus Puggaard-Rode <rasmus.puggaard-rode@ling-phil.ox.ac.uk>
Repository: CRAN
Date/Publication: 2026-07-20 15:30:07 UTC
