# Generated by roxygen2: do not edit by hand

export(conv2sc)
export(draw_arrow)
export(draw_lines)
export(draw_rectangle)
export(draw_spectralslice)
export(emupicture)
export(formantplot)
export(intensity_overlay)
export(intensityplot)
export(make_TextGrid)
export(make_annot)
export(pitch_overlay)
export(pitchplot)
export(praatanimation)
export(praatpicture)
export(shiny_praatpicture)
export(specplot)
export(talking_praatpicture)
export(tg_createTier)
export(tg_stylize)
export(tgplot)
export(waveplot)
